@@ -310,7 +310,7 @@ void Http::SendMethod(const char *method,const char *efile)
 {
    xstring& stripped_hostname=xstring::get_tmp(hostname);
    stripped_hostname.truncate_at('%');
-   xstring& ehost=url::encode(stripped_hostname,URL_HOST_UNSAFE);
+   xstring& ehost=url::encode(xidna_to_ascii(stripped_hostname),URL_HOST_UNSAFE);
    if(portname) {
       ehost.append(':');
       ehost.append(url::encode(portname,URL_PORT_UNSAFE));
@@ -1965,6 +1965,8 @@ get_again:
 	 if(chunked)
 	    chunk_pos+=size1;
 	 bytes_received+=size1;
+	 if(inflate->Error())
+	    SetError(FATAL,inflate->ErrorText());
       }
       inflate->Get(&buf1,&size1);
       src_buf=inflate.get_non_const();
@@ -1986,7 +1988,7 @@ get_again:
    }
    if(size>size1)
       size=size1;
-   buf->MoveDataHere(src_buf,size);
+   size=buf->MoveDataHere(src_buf,size);
    _UpdatePos(size);
    return size;
 }
@@ -2600,6 +2602,8 @@ bool Http::CompressedContentEncoding() const
 }
 bool Http::CompressedContentType() const
 {
+   if(file.ends_with(".gz") || file.ends_with(".Z") || file.ends_with(".tgz"))
+      return true;
    static const char app[]="application/";
    return entity_content_type && entity_content_type.begins_with(app)
       && IsCompressed(entity_content_type+sizeof(app)-1);
