@@ -31,6 +31,7 @@
 #include "rmJob.h"
 #include "mvJob.h"
 #include "ChmodJob.h"
+#include "mkdirJob.h"
 #include "misc.h"
 #include "plural.h"
 #include "FindJob.h"
@@ -57,6 +58,8 @@ xstring& MirrorJob::FormatStatus(xstring& s,int v,const char *tab)
    case(TARGET_REMOVE_OLD):
    case(TARGET_REMOVE_OLD_FIRST):
    case(TARGET_CHMOD):
+   case(TARGET_MKDIR):
+   case(SOURCE_REMOVING_SAME):
    case(LAST_EXEC):
       break;
 
@@ -131,6 +134,8 @@ void  MirrorJob::ShowRunStatus(const SMTaskRef<StatusLine>& s)
    case(TARGET_REMOVE_OLD):
    case(TARGET_REMOVE_OLD_FIRST):
    case(TARGET_CHMOD):
+   case(TARGET_MKDIR):
+   case(SOURCE_REMOVING_SAME):
    case(FINISHING):
    case(DONE):
    case(LAST_EXEC):
@@ -221,7 +226,8 @@ void MirrorJob::TransferFinished(Job *j)
       const xstring& rate=Speedometer::GetStrProper(j->GetTransferRate());
       if(rate.length()>0)
 	 finished.append(" (").append(rate).append(')');
-      Report(_("Finished %s"),finished.get());
+      if(!(FlagSet(SCAN_ALL_FIRST) && finished.begins_with("mirror")))
+	 Report(_("Finished %s"),finished.get());
    }
    JobFinished(j);
    if(transfer_count==0)
@@ -342,6 +348,8 @@ void  MirrorJob::HandleFile(FileInfo *file)
 	       args.Append("-e");
 	    if(FlagSet(ASCII))
 	       args.Append("-a");
+	    if(remove_source_files)
+	       args.Append("-E");
 	    args.Append("-O");
 	    args.Append(target_is_local?target_dir.get()
 			:target_session->GetConnectURL());
@@ -460,8 +468,12 @@ void  MirrorJob::HandleFile(FileInfo *file)
 
 	 mj->max_error_count=max_error_count;
 
-	 if(verbose_report>=3)
-	    Report(_("Mirroring directory `%s'"),mj->target_relative_dir.get());
+	 if(verbose_report>=3) {
+	    if(FlagSet(SCAN_ALL_FIRST))
+	       Report(_("Scanning directory `%s'"),mj->target_relative_dir.get());
+	    else
+	       Report(_("Mirroring directory `%s'"),mj->target_relative_dir.get());
+	 }
 
 	 break;
       }
@@ -582,6 +594,12 @@ void  MirrorJob::InitSets(const FileSet *source,const FileSet *dest)
    if(size_range)
       to_transfer->SubtractSizeOutside(size_range);
 
+   if(FlagSet(SCAN_ALL_FIRST)) {
+      to_mkdir=new FileSet(to_transfer);
+      to_mkdir->SubtractNotDirs();
+      to_mkdir->SubtractAny(dest);
+   }
+
    if(flags&NO_RECURSION)
       to_transfer->SubtractDirs();
 
@@ -612,8 +630,13 @@ void  MirrorJob::InitSets(const FileSet *source,const FileSet *dest)
       to_transfer->ReverseSort();
 
    int dir_count=0;
-   to_transfer->Count(&dir_count,NULL,NULL,NULL);
-   only_dirs = (dir_count==to_transfer->count());
+   if(to_mkdir) {
+      to_mkdir->Count(&dir_count,NULL,NULL,NULL);
+      only_dirs = (dir_count==to_mkdir->count());
+   } else {
+      to_transfer->Count(&dir_count,NULL,NULL,NULL);
+      only_dirs = (dir_count==to_transfer->count());
+   }
 }
 
 void MirrorJob::HandleChdir(FileAccessRef& session, int &redirections)
@@ -628,29 +651,32 @@ void MirrorJob::HandleChdir(FileAccessRef& session, int &redirections)
 	 // cd to another url.
 	 const char *loc_c=session->GetNewLocation();
 	 int max_redirections=ResMgr::Query("xfer:max-redirections",0);
-	 if(loc_c && max_redirections>0 && last_char(loc_c)=='/')
+	 if(loc_c && max_redirections>0)
 	 {
 	    if(++redirections>max_redirections)
 	       goto cd_err_normal;
 	    eprintf(_("%s: received redirection to `%s'\n"),"mirror",loc_c);
 
 	    char *loc=alloca_strdup(loc_c);
-	    session->Close(); // loc_c is no longer valid.
-
 	    ParsedURL u(loc,true);
 
 	    if(!u.proto)
 	    {
-	       session->Chdir(url::decode(loc));
+	       bool is_file=(last_char(loc)!='/');
+	       FileAccess::Path new_cwd(session->GetNewCwd());
+	       new_cwd.Change(0,is_file,loc);
+	       session->PathVerify(new_cwd);
+	       session->Roll();
 	       return;
 	    }
+	    session->Close(); // loc_c is no longer valid.
 	    session=FA::New(&u);
 	    session->Chdir(u.path);
 	    return;
 	 }
       }
    cd_err_normal:
-      if(session==target_session && script_only)
+      if(session==target_session && (script_only || FlagSet(SCAN_ALL_FIRST)))
       {
 	 char *dir=alloca_strdup(session->GetFile());
 	 session->Close();
@@ -751,7 +777,7 @@ int   MirrorJob::Do()
 
    pre_MAKE_TARGET_DIR:
    {
-      if(!strcmp(target_dir,".") || !strcmp(target_dir,".."))
+      if(!strcmp(target_dir,".") || !strcmp(target_dir,"..") || (FlagSet(SCAN_ALL_FIRST) && parent_mirror))
 	 create_target_dir=false;
       if(!create_target_dir)
 	 goto pre_CHANGING_DIR_TARGET;
@@ -865,8 +891,35 @@ int   MirrorJob::Do()
       }
 
       // now we have both target and source file sets.
-      stats.dirs++;
+      if(parent_mirror)
+	 stats.dirs++;
 
+      if(FlagSet(SCAN_ALL_FIRST) && parent_mirror)
+      {
+	 source_set->PrependPath(source_relative_dir);
+	 if(root_mirror->source_set_recursive)
+	    root_mirror->source_set_recursive->Merge(source_set);
+	 else
+	    root_mirror->source_set_recursive=source_set.borrow();
+	 if(target_set) {
+	    target_set->PrependPath(target_relative_dir);
+	    if(root_mirror->target_set_recursive)
+	       root_mirror->target_set_recursive->Merge(target_set);
+	    else
+	       root_mirror->target_set_recursive=target_set.borrow();
+	 }
+	 root_mirror->stats.dirs++;
+	 goto pre_DONE;
+      }
+
+      if(source_set_recursive) {
+	 source_set->Merge(source_set_recursive);
+	 source_set_recursive=0;
+      }
+      if(target_set_recursive) {
+	 target_set->Merge(target_set_recursive);
+	 target_set_recursive=0;
+      }
       InitSets(source_set,target_set);
 
       to_transfer->CountBytes(&bytes_to_transfer);
@@ -881,6 +934,43 @@ int   MirrorJob::Do()
       set_state(TARGET_REMOVE_OLD_FIRST);
       goto TARGET_REMOVE_OLD_FIRST_label;
 
+   pre_TARGET_MKDIR:
+      if(!to_mkdir)
+	 goto pre_WAITING_FOR_TRANSFER;
+      to_mkdir->rewind();
+      set_state(TARGET_MKDIR);
+      m=MOVED;
+      /*fallthrough*/
+   case(TARGET_MKDIR):
+      while((j=FindDoneAwaitedJob())!=0)
+      {
+	 JobFinished(j);
+	 m=MOVED;
+      }
+      if(max_error_count>0 && stats.error_count>=max_error_count)
+	 goto pre_FINISHING;
+      while(transfer_count<parallel && state==TARGET_MKDIR)
+      {
+	 file=to_mkdir->curr();
+	 if(!file)
+	    goto pre_WAITING_FOR_TRANSFER;
+	 to_mkdir->next();
+	 if(!file->TypeIs(file->DIRECTORY))
+	    continue;
+	 if(script)
+	    fprintf(script,"mkdir %s\n",target_session->GetFileURL(file->name));
+	 if(!script_only)
+	 {
+	    ArgV *a=new ArgV("mkdir");
+	    a->Append(file->name);
+	    mkdirJob *mkj=new mkdirJob(target_session->Clone(),a);
+	    mkj->cmdline.set_allocated(a->Combine());
+	    JobStarted(mkj);
+	    m=MOVED;
+	 }
+      }
+      break;
+
    pre_WAITING_FOR_TRANSFER:
       to_transfer->rewind();
       set_state(WAITING_FOR_TRANSFER);
@@ -911,7 +1001,7 @@ int   MirrorJob::Do()
 	       // if we have not created any subdirs and there are only subdirs,
 	       // then the directory would be empty - skip it.
 	       if(FlagSet(NO_EMPTY_DIRS) && stats.dirs==0 && only_dirs)
-		  goto pre_FINISHING;
+		  goto pre_FINISHING_FIX_LOCAL;
 
 	       transfer_count+=root_transfer_count;
 	       goto pre_MAKE_TARGET_DIR;
@@ -959,7 +1049,7 @@ int   MirrorJob::Do()
 	       break;
 	    if(state==TARGET_REMOVE_OLD)
 	       goto pre_TARGET_CHMOD;
-	    goto pre_WAITING_FOR_TRANSFER;
+	    goto pre_TARGET_MKDIR;
 	 }
 	 const char *target_name_rel=dir_file(target_relative_dir,file->name);
 	 target_name_rel=alloca_strdup(target_name_rel);
@@ -979,7 +1069,7 @@ int   MirrorJob::Do()
 	    ArgV args("rm");
 	    if(file->defined&file->TYPE && file->filetype==file->DIRECTORY)
 	    {
-	       if(flags&NO_RECURSION)
+	       if(FlagSet(NO_RECURSION) && !FlagSet(SCAN_ALL_FIRST))
 		  args.setarg(0,"rmdir");
 	       else
 		  args.Append("-r");
@@ -998,7 +1088,7 @@ int   MirrorJob::Do()
 	    JobStarted(j);
 	    if(file->defined&file->TYPE && file->filetype==file->DIRECTORY)
 	    {
-	       if(flags&NO_RECURSION)
+	       if(FlagSet(NO_RECURSION) && !FlagSet(SCAN_ALL_FIRST))
 	       {
 		  args->setarg(0,"rmdir");
 		  j->Rmdir();
@@ -1016,7 +1106,7 @@ int   MirrorJob::Do()
 
    pre_TARGET_CHMOD:
       if(flags&NO_PERMS)
-	 goto pre_FINISHING;
+	 goto pre_FINISHING_FIX_LOCAL;
 
       to_transfer->rewind();
       set_state(TARGET_CHMOD);
@@ -1034,7 +1124,7 @@ int   MirrorJob::Do()
       {
 	 file=to_transfer->curr();
 	 if(!file)
-	    goto pre_FINISHING;
+	    goto pre_FINISHING_FIX_LOCAL;
 	 to_transfer->next();
 	 if((file->defined&file->TYPE) && file->filetype==file->SYMLINK)
 	    continue;
@@ -1077,7 +1167,7 @@ int   MirrorJob::Do()
       }
       break;
 
-   pre_FINISHING:
+   pre_FINISHING_FIX_LOCAL:
       if(target_is_local && !script_only)     // FIXME
       {
 	 to_transfer->LocalUtime(target_dir,/*only_dirs=*/true);
@@ -1088,6 +1178,9 @@ int   MirrorJob::Do()
 	 if(FlagSet(ALLOW_CHOWN) && same)
 	    same->LocalChown(target_dir);
       }
+      if(remove_source_files)
+	 goto pre_SOURCE_REMOVING_SAME;
+   pre_FINISHING:
       set_state(FINISHING);
       m=MOVED;
       /*fallthrough*/
@@ -1116,6 +1209,50 @@ int   MirrorJob::Do()
 	 }
       }
       goto pre_DONE;
+
+   pre_SOURCE_REMOVING_SAME:
+      same->rewind();
+      set_state(SOURCE_REMOVING_SAME);
+      m=MOVED;
+      /*fallthrough*/
+   case(SOURCE_REMOVING_SAME):
+      while((j=FindDoneAwaitedJob())!=0)
+      {
+	 JobFinished(j);
+	 m=MOVED;
+      }
+      if(max_error_count>0 && stats.error_count>=max_error_count)
+	 goto pre_FINISHING;
+      while(transfer_count<parallel && state==SOURCE_REMOVING_SAME)
+      {
+	 file=same->curr();
+	 same->next();
+	 if(!file)
+	    goto pre_FINISHING;
+	 if(file->TypeIs(file->DIRECTORY))
+	    continue;
+	 const char *source_name_rel=dir_file(source_relative_dir,file->name);
+	 source_name_rel=alloca_strdup(source_name_rel);
+	 if(script)
+	 {
+	    ArgV args("rm");
+	    args.Append(source_session->GetFileURL(file->name));
+	    xstring_ca cmd(args.CombineQuoted());
+	    fprintf(script,"%s\n",cmd.get());
+	 }
+	 if(!script_only)
+	 {
+	    ArgV *args=new ArgV("rm");
+	    args->Append(file->name);
+	    args->seek(1);
+	    rmJob *j=new rmJob(source_session->Clone(),args);
+	    j->cmdline.set_allocated(args->Combine());
+	    JobStarted(j);
+	 }
+	 Report(_("Removing source file `%s'"),source_name_rel);
+      }
+      break;
+
    case(LAST_EXEC):
       while((j=FindDoneAwaitedJob())!=0)
       {
@@ -1441,6 +1578,7 @@ CMD(mirror)
       OPT_NO_EMPTY_DIRS,
       OPT_DEPTH_FIRST,
       OPT_ASCII,
+      OPT_SCAN_ALL_FIRST,
    };
    static const struct option mirror_opts[]=
    {
@@ -1488,6 +1626,7 @@ CMD(mirror)
       {"ascii",no_argument,0,OPT_ASCII},
       {"target-directory",required_argument,0,'O'},
       {"destination-directory",required_argument,0,'O'},
+      {"scan-all-first",no_argument,0,OPT_SCAN_ALL_FIRST},
       {0}
    };
 
@@ -1543,6 +1682,7 @@ CMD(mirror)
 	 break;
       case('r'):
 	 flags|=MirrorJob::NO_RECURSION;
+	 flags&=~MirrorJob::SCAN_ALL_FIRST;
 	 break;
       case('n'):
 	 flags|=MirrorJob::ONLY_NEWER;
@@ -1594,6 +1734,7 @@ CMD(mirror)
       {
 	 // mirror for a single file (or glob pattern).
 	 flags|=MirrorJob::NO_RECURSION;
+	 flags&=~MirrorJob::SCAN_ALL_FIRST;
 	 MirrorJob::AddPattern(exclude,'I',basename_ptr(optarg));
 	 source_dir=dirname(optarg);
 	 source_dir=alloca_strdup(source_dir); // save the temp string
@@ -1679,6 +1820,9 @@ CMD(mirror)
       case(OPT_DEPTH_FIRST):
 	 flags|=MirrorJob::DEPTH_FIRST;
 	 break;
+      case(OPT_SCAN_ALL_FIRST):
+	 flags|=MirrorJob::SCAN_ALL_FIRST|MirrorJob::DEPTH_FIRST;
+	 break;
       case(OPT_ASCII):
 	 flags|=MirrorJob::ASCII|MirrorJob::IGNORE_SIZE;
 	 break;
