@@ -1937,13 +1937,17 @@ CMD(jobs)
 {
    int opt;
    int v=1;
-   while((opt=args->getopt("+v"))!=EOF)
+   bool recursion=true;
+   while((opt=args->getopt("+vr"))!=EOF)
    {
       switch(opt)
       {
       case('v'):
 	 v++;
 	 break;
+      case('r'):
+	 recursion=false;
+	 break;
       case('?'):
          // xgettext:c-format
 	 eprintf(_("Usage: %s [-v] [-v] ...\n"),args->a0());
@@ -1971,7 +1975,10 @@ CMD(jobs)
 	    exit_code=1;
 	    continue;
 	 }
-	 j->FormatOneJob(s,v);
+	 if(recursion)
+	    j->FormatOneJobRecursively(s,v);
+	 else
+	    j->FormatOneJob(s,v);
       }
    }
    if(exit_code)
@@ -2039,13 +2046,12 @@ CMD(debug)
 	 debug_file_name=optarg;
 	 if(fd!=-1)
 	    close(fd);
-	 fd=open(debug_file_name,O_WRONLY|O_CREAT|O_APPEND,0600);
+	 fd=open(debug_file_name,O_WRONLY|O_CREAT|O_APPEND|O_NONBLOCK,0600);
 	 if(fd==-1)
 	 {
 	    perror(debug_file_name);
 	    return 0;
 	 }
-	 fcntl(fd,F_SETFL,O_NONBLOCK);
 	 fcntl(fd,F_SETFD,FD_CLOEXEC);
 	 break;
       case 'p':
@@ -2977,6 +2983,7 @@ CMD(du)
 	 break;
       case 'b':
 	 blocksize = 1;
+	 human_opts = 0;
 	 break;
       case 'c':
 	 print_totals=true;
@@ -2994,18 +3001,18 @@ CMD(du)
 	 file_count=true;
 	 break;
       case 'h':
-	 blocksize=1;
-	 human_opts|=human_autoscale|human_SI|human_base_1024;
+	 human_opts |= human_autoscale|human_SI|human_base_1024;
 	 break;
       case 'H':
-	 blocksize=1;
 	 human_opts |= human_autoscale|human_SI;
 	 break;
       case 'k': /* the default; here for completeness */
 	 blocksize = 1024;
+	 human_opts = 0;
 	 break;
       case 'm':
 	 blocksize = 1024*1024;
+	 human_opts = 0;
 	 break;
       case 's':
 	 summarize_only = true;
