@@ -31,6 +31,7 @@
 ResType pget_vars[] = {
    {"pget:save-status",	"10s",   ResMgr::TimeIntervalValidate,ResMgr::NoClosure},
    {"pget:default-n",   "5",	 ResMgr::UNumberValidate,ResMgr::NoClosure},
+   {"pget:min-chunk-size", "1M", ResMgr::UNumberValidate,ResMgr::NoClosure},
    {0}
 };
 ResDecls pget_vars_register(pget_vars);
@@ -39,7 +40,6 @@ ResDecls pget_vars_register(pget_vars);
 #define MIN(a,b) ((a)<(b)?(a):(b))
 
 #define super CopyJob
-#define min_chunk_size 0x10000
 
 int pgetJob::Do()
 {
@@ -139,6 +139,11 @@ int pgetJob::Do()
 	 return m;
       }
 
+      // Make sure the destination file is open before starting chunks,
+      // it disables temp-name creation in the chunk's Init.
+      if(c->put->GetLocal()->getfd()==-1)
+	 return m;
+
       c->put->NeedSeek(); // seek before writing
 
       if(pget_cont)
@@ -303,7 +308,6 @@ void pgetJob::free_chunks()
 	 chunks_bytes+=chunks[i]->GetBytesCount();
       chunks.unset();
    }
-   cmdline.set(0);
 }
 
 pgetJob::pgetJob(FileCopy *c1,const char *n,int m)
@@ -495,6 +499,7 @@ void pgetJob::InitChunks(off_t offset,off_t size)
 {
    /* initialize chunks */
    off_t chunk_size=(size-offset)/max_chunks;
+   int min_chunk_size=ResMgr::Query("pget:min-chunk-size",0);
    if(chunk_size<min_chunk_size)
       chunk_size=min_chunk_size;
    int num_of_chunks=(size-offset)/chunk_size-1;
