@@ -197,6 +197,7 @@ static ResType lftp_vars[] = {
    {"ftp:auto-sync-mode",	 "",	  ResMgr::ERegExpValidate,0},
    {"ftp:auto-passive-mode",	 "yes",   ResMgr::BoolValidate,0},
    {"ftp:bind-data-socket",	 "yes",   ResMgr::BoolValidate,0},
+   {"ftp:catch-size",		 "yes",	  ResMgr::BoolValidate,0},
    {"ftp:charset",		 "",	  ResMgr::CharsetValidate,0},
    {"ftp:client",		 PACKAGE "/" VERSION,0,0},
    {"ftp:device-prefix",	 "no",	  ResMgr::BoolValidate,0},
@@ -336,6 +337,7 @@ static ResType lftp_vars[] = {
    {"mirror:dereference",	 "no",    ResMgr::BoolValidate,ResMgr::NoClosure},
    {"mirror:skip-noaccess",	 "no",    ResMgr::BoolValidate,ResMgr::NoClosure},
    {"mirror:no-empty-dirs",	 "no",	  ResMgr::BoolValidate,ResMgr::NoClosure},
+   {"mirror:require-source",	 "no",	  ResMgr::BoolValidate,ResMgr::NoClosure},
 
    {"sftp:max-packets-in-flight","16",	  ResMgr::UNumberValidate,0},
    {"sftp:protocol-version",	 "6",	  ResMgr::UNumberValidate,0},
@@ -378,6 +380,8 @@ static ResType lftp_vars[] = {
    {"xfer:auto-rename",		 "no",	  ResMgr::BoolValidate,ResMgr::NoClosure},
    {"xfer:log-file",              "",     ResMgr::FileCreatable,ResMgr::NoClosure},
    {"xfer:max-log-size",	 "1M",	  ResMgr::UNumberValidate,ResMgr::NoClosure},
+   {"xfer:use-temp-file",	 "no",	  ResMgr::BoolValidate,ResMgr::NoClosure},
+   {"xfer:temp-file-name",	 ".in.*", 0,ResMgr::NoClosure},
 
 #if USE_SSL
    {"ssl:ca-file",		 "",	  ResMgr::FileReadable,ResMgr::NoClosure},
